@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004-2007 Jay Sorg
+   Copyright (c) 2004-2012 Jay Sorg
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -24,20 +24,21 @@
 #if !defined(ARCH_H)
 #define ARCH_H
 
+#if !(defined(L_ENDIAN) || defined(B_ENDIAN))
 /* check endianess */
-#if defined(__sparc__) || defined(__PPC__) || defined(__ppc__)
+#if defined(__sparc__) || defined(__PPC__) || defined(__ppc__) || \
+    defined(__hppa__)
 #define B_ENDIAN
-#elif __BYTE_ORDER == __LITTLE_ENDIAN
+#else
 #define L_ENDIAN
-#elif __BYTE_ORDER == __BIG_ENDIAN
-#define B_ENDIAN
 #endif
 /* check if we need to align data */
 #if defined(__sparc__) || defined(__alpha__) || defined(__hppa__) || \
     defined(__AIX__) || defined(__PPC__) || defined(__mips__) || \
-    defined(__ia64__) || defined(__ppc__)
+    defined(__ia64__) || defined(__ppc__) || defined(__arm__)
 #define NEED_ALIGN
 #endif
+#endif
 
 /* defines for thread creation factory functions */
 #if defined(_WIN32)
@@ -48,7 +49,7 @@
 #define THREAD_CC
 #endif
 
-#if defined(__BORLANDC__)
+#if defined(__BORLANDC__) || defined(_WIN32)
 #define APP_CC __fastcall
 #define DEFAULT_CC __cdecl
 #else
@@ -57,10 +58,14 @@
 #endif
 
 #if defined(_WIN32)
+#if defined(__BORLANDC__)
 #define EXPORT_CC _export __cdecl
 #else
 #define EXPORT_CC
 #endif
+#else
+#define EXPORT_CC
+#endif
 
 typedef char ti8;
 typedef unsigned char tui8;
@@ -71,6 +76,28 @@ typedef signed short tsi16;
 typedef int ti32;
 typedef unsigned int tui32;
 typedef signed int tsi32;
+#if defined(_WIN64)
+/* Microsoft's VC++ compiler uses the more backwards-compatible LLP64 model.
+   Most other 64 bit compilers(Solaris, AIX, HP, Linux, Mac OS X) use
+   the LP64 model.
+   long is 32 bits in LLP64 model, 64 bits in LP64 model. */
+typedef __int64 tbus;
+#else
 typedef long tbus;
+#endif
+typedef tbus thandle;
+typedef tbus tintptr;
+/* wide char, socket */
+#if defined(_WIN32)
+typedef unsigned short twchar;
+typedef unsigned int tsock;
+typedef unsigned __int64 tui64;
+typedef signed __int64 tsi64;
+#else
+typedef int twchar;
+typedef int tsock;
+typedef unsigned long long tui64;
+typedef signed long long tsi64;
+#endif
 
 #endif
