@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2004-2007
+   Copyright (C) Jay Sorg 2004-2010
 
 */
 
@@ -23,6 +23,10 @@
 
 #include "arch.h"
 
+int
+ssl_init(void);
+int
+ssl_finish(void);
 void* APP_CC
 ssl_rc4_info_create(void);
 void APP_CC
@@ -54,5 +58,8 @@ ssl_md5_complete(void* md5_info, char* data);
 int APP_CC
 ssl_mod_exp(char* out, int out_len, char* in, int in_len,
             char* mod, int mod_len, char* exp, int exp_len);
+int APP_CC
+ssl_gen_key_xrdp1(int key_size_in_bits, char* exp, int exp_len,
+                  char* mod, int mod_len, char* pri, int pri_len);
 
 #endif
