@@ -202,6 +202,11 @@ trans_force_read_s(struct trans* self, struct stream* in_s, int size)
   }
   while (size > 0)
   {
+    /* make sure stream has room */
+    if ((in_s->end + size) > (in_s->data + in_s->size))
+    {
+      return 1;
+    }
     rcvd = g_tcp_recv(self->sck, in_s->end, size, 0);
     if (rcvd == -1)
     {
