@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2004-2007
+   Copyright (C) Jay Sorg 2004-2010
 
    libxrdp header
 
@@ -23,8 +23,12 @@
 #if !defined(LIBXRDP_H)
 #define LIBXRDP_H
 
+#if defined(HAVE_CONFIG_H)
+#include "config_ac.h"
+#endif
 #include "arch.h"
 #include "parse.h"
+#include "trans.h"
 #include "xrdp_constants.h"
 #include "defines.h"
 #include "os_calls.h"
@@ -32,12 +36,12 @@
 #include "list.h"
 #include "file.h"
 #include "libxrdpinc.h"
+#include "file_loc.h"
 
 /* tcp */
 struct xrdp_tcp
 {
-  int sck;
-  int sck_closed;
+  struct trans* trans;
   struct xrdp_iso* iso_layer; /* owner */
 };
 
@@ -54,7 +58,6 @@ struct mcs_channel_item
   char name[16];
   int flags;
   int chanid;
-  struct stream* in_s;
 };
 
 /* mcs */
@@ -211,7 +214,7 @@ struct xrdp_orders
 
 /* xrdp_tcp.c */
 struct xrdp_tcp* APP_CC
-xrdp_tcp_create(struct xrdp_iso* owner, int sck);
+xrdp_tcp_create(struct xrdp_iso* owner, struct trans* trans);
 void APP_CC
 xrdp_tcp_delete(struct xrdp_tcp* self);
 int APP_CC
@@ -223,7 +226,7 @@ xrdp_tcp_send(struct xrdp_tcp* self, struct stream* s);
 
 /* xrdp_iso.c */
 struct xrdp_iso* APP_CC
-xrdp_iso_create(struct xrdp_mcs* owner, int sck);
+xrdp_iso_create(struct xrdp_mcs* owner, struct trans* trans);
 void APP_CC
 xrdp_iso_delete(struct xrdp_iso* self);
 int APP_CC
@@ -237,7 +240,7 @@ xrdp_iso_incoming(struct xrdp_iso* self);
 
 /* xrdp_mcs.c */
 struct xrdp_mcs* APP_CC
-xrdp_mcs_create(struct xrdp_sec* owner, int sck,
+xrdp_mcs_create(struct xrdp_sec* owner, struct trans* trans,
                 struct stream* client_mcs_data,
                 struct stream* server_mcs_data);
 void APP_CC
@@ -255,7 +258,7 @@ xrdp_mcs_disconnect(struct xrdp_mcs* self);
 
 /* xrdp_sec.c */
 struct xrdp_sec* APP_CC
-xrdp_sec_create(struct xrdp_rdp* owner, int sck, int crypt_level,
+xrdp_sec_create(struct xrdp_rdp* owner, struct trans* trans, int crypt_level,
                 int channel_code);
 void APP_CC
 xrdp_sec_delete(struct xrdp_sec* self);
@@ -276,7 +279,7 @@ xrdp_sec_disconnect(struct xrdp_sec* self);
 
 /* xrdp_rdp.c */
 struct xrdp_rdp* APP_CC
-xrdp_rdp_create(struct xrdp_session* session, int sck);
+xrdp_rdp_create(struct xrdp_session* session, struct trans* trans);
 void APP_CC
 xrdp_rdp_delete(struct xrdp_rdp* self);
 int APP_CC
@@ -379,6 +382,9 @@ int APP_CC
 xrdp_orders_send_bitmap2(struct xrdp_orders* self,
                          int width, int height, int bpp, char* data,
                          int cache_id, int cache_idx);
+int APP_CC
+xrdp_orders_send_brush(struct xrdp_orders* self, int width, int height,
+                       int bpp, int type, int size, char* data, int cache_id);
 
 /* xrdp_bitmap_compress.c */
 int APP_CC
@@ -395,7 +401,8 @@ xrdp_channel_delete(struct xrdp_channel* self);
 int APP_CC
 xrdp_channel_init(struct xrdp_channel* self, struct stream* s);
 int APP_CC
-xrdp_channel_send(struct xrdp_channel* self, struct stream* s, int channel_id);
+xrdp_channel_send(struct xrdp_channel* self, struct stream* s, int channel_id,
+                  int total_data_len, int flags);
 int APP_CC
 xrdp_channel_process(struct xrdp_channel* self, struct stream* s,
                      int chanid);
