@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2004-2007
+   Copyright (C) Jay Sorg 2004-2010
 
    orders
 
@@ -127,6 +127,10 @@ xrdp_orders_send(struct xrdp_orders* self)
 int APP_CC
 xrdp_orders_force_send(struct xrdp_orders* self)
 {
+  if (self == 0)
+  {
+    return 1;
+  }
   if ((self->order_level > 0) && (self->order_count > 0))
   {
     s_mark_end(self->out_s);
@@ -173,7 +177,7 @@ xrdp_orders_check(struct xrdp_orders* self, int max_size)
       return 0;
     }
   }
-  size = self->out_s->p - self->order_count_ptr;
+  size = (int)(self->out_s->p - self->order_count_ptr);
   if ((size < 0) || (size > max_packet_size))
   {
     return 1;
@@ -196,10 +200,10 @@ xrdp_orders_last_bounds(struct xrdp_orders* self, struct xrdp_rect* rect)
   {
     return 0;
   }
-  if (rect->left == self->orders_state.clip_left &&
-      rect->top == self->orders_state.clip_top &&
-      rect->right == self->orders_state.clip_right &&
-      rect->bottom == self->orders_state.clip_bottom)
+  if ((rect->left == self->orders_state.clip_left) &&
+      (rect->top == self->orders_state.clip_top) &&
+      (rect->right == self->orders_state.clip_right) &&
+      (rect->bottom == self->orders_state.clip_bottom))
   {
     return 1;
   }
@@ -328,6 +332,55 @@ xrdp_orders_out_bounds(struct xrdp_orders* self, struct xrdp_rect* rect)
   return 0;
 }
 
+/*****************************************************************************/
+/* returns error */
+static int APP_CC
+xrdp_order_pack_small_or_tiny(struct xrdp_orders* self,
+                              char* order_flags_ptr, int orders_flags,
+                              char* present_ptr, int present,
+                              int present_size)
+{
+  int move_up_count = 0;
+  int index = 0;
+  int size = 0;
+  int keep_looking = 1;
+
+  move_up_count = 0;
+  keep_looking = 1;
+  for (index = present_size - 1; index >= 0; index--)
+  {
+    if (keep_looking)
+    {
+      if (((present >> (index * 8)) & 0xff) == 0)
+      {
+        move_up_count++;
+      }
+      else
+      {
+        keep_looking = 0;
+      }
+    }
+    present_ptr[index] = present >> (index * 8);
+  }
+  if (move_up_count > 0)
+  {
+    /* move_up_count should be 0, 1, 2, or 3
+       shifting it 6 will make it RDP_ORDER_TINY(0x80) or
+       RDP_ORDER_SMALL(0x40) or both */
+    orders_flags |= move_up_count << 6;
+    size = (int)(self->out_s->p - present_ptr);
+    size -= present_size;
+    for (index = 0; index < size; index++)
+    {
+      present_ptr[index + (present_size - move_up_count)] =
+        present_ptr[index + present_size];
+    }
+    self->out_s->p -= move_up_count;
+  }
+  order_flags_ptr[0] = orders_flags;
+  return 0;
+}
+
 /*****************************************************************************/
 /* returns error */
 /* send a solid rect to client */
@@ -340,6 +393,7 @@ xrdp_orders_rect(struct xrdp_orders* self, int x, int y, int cx, int cy,
   int vals[8];
   int present;
   char* present_ptr;
+  char* order_flags_ptr;
 
   xrdp_orders_check(self, 23);
   self->order_count++;
@@ -374,14 +428,17 @@ xrdp_orders_rect(struct xrdp_orders* self, int x, int y, int cx, int cy,
   {
     order_flags |= RDP_ORDER_DELTA;
   }
-  out_uint8(self->out_s, order_flags)
+  /* order_flags, set later, 1 byte */
+  order_flags_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if (order_flags & RDP_ORDER_CHANGE)
   {
     out_uint8(self->out_s, self->orders_state.last_order);
   }
   present = 0;
-  present_ptr = self->out_s->p; /* hold 1 byte present pointer */
-  out_uint8s(self->out_s, 1)
+  /* present, set later, 1 byte */
+  present_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if ((order_flags & RDP_ORDER_BOUNDS) &&
       !(order_flags & RDP_ORDER_LASTBOUNDS))
   {
@@ -460,7 +517,8 @@ xrdp_orders_rect(struct xrdp_orders* self, int x, int y, int cx, int cy,
                  (self->orders_state.rect_color & 0x00ffff) | (color & 0xff0000);
     out_uint8(self->out_s, color >> 16);
   }
-  present_ptr[0] = present;
+  xrdp_order_pack_small_or_tiny(self, order_flags_ptr, order_flags,
+                                present_ptr, present, 1);
   return 0;
 }
 
@@ -473,10 +531,11 @@ xrdp_orders_screen_blt(struct xrdp_orders* self, int x, int y,
                        int cx, int cy, int srcx, int srcy,
                        int rop, struct xrdp_rect* rect)
 {
-  int order_flags;
-  int vals[12];
-  int present;
-  char* present_ptr;
+  int order_flags = 0;
+  int vals[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+  int present = 0;
+  char* present_ptr = (char *)NULL;
+  char* order_flags_ptr = (char *)NULL;
 
   xrdp_orders_check(self, 25);
   self->order_count++;
@@ -515,14 +574,17 @@ xrdp_orders_screen_blt(struct xrdp_orders* self, int x, int y,
   {
     order_flags |= RDP_ORDER_DELTA;
   }
-  out_uint8(self->out_s, order_flags);
+  /* order_flags, set later, 1 byte */
+  order_flags_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if (order_flags & RDP_ORDER_CHANGE)
   {
     out_uint8(self->out_s, self->orders_state.last_order);
   }
   present = 0;
-  present_ptr = self->out_s->p; /* hold 1 byte present pointer */
-  out_uint8s(self->out_s, 1)
+  /* present, set later, 1 byte */
+  present_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if ((order_flags & RDP_ORDER_BOUNDS) &&
       !(order_flags & RDP_ORDER_LASTBOUNDS))
   {
@@ -604,15 +666,16 @@ xrdp_orders_screen_blt(struct xrdp_orders* self, int x, int y,
     present |= 0x40;
     if (order_flags & RDP_ORDER_DELTA)
     {
-      out_uint8(self->out_s, srcy - self->orders_state.scr_blt_srcy)
+      out_uint8(self->out_s, srcy - self->orders_state.scr_blt_srcy);
     }
     else
     {
-      out_uint16_le(self->out_s, srcy)
+      out_uint16_le(self->out_s, srcy);
     }
     self->orders_state.scr_blt_srcy = srcy;
   }
-  present_ptr[0] = present;
+  xrdp_order_pack_small_or_tiny(self, order_flags_ptr, order_flags,
+                                present_ptr, present, 1);
   return 0;
 }
 
@@ -627,9 +690,10 @@ xrdp_orders_pat_blt(struct xrdp_orders* self, int x, int y,
                     struct xrdp_rect* rect)
 {
   int order_flags;
-  int vals[8];
   int present;
+  int vals[8];
   char* present_ptr;
+  char* order_flags_ptr;
   struct xrdp_brush blank_brush;
 
   xrdp_orders_check(self, 39);
@@ -665,15 +729,17 @@ xrdp_orders_pat_blt(struct xrdp_orders* self, int x, int y,
   {
     order_flags |= RDP_ORDER_DELTA;
   }
-  out_uint8(self->out_s, order_flags);
+  /* order_flags, set later, 1 byte */
+  order_flags_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if (order_flags & RDP_ORDER_CHANGE)
   {
     out_uint8(self->out_s, self->orders_state.last_order);
   }
   present = 0;
-  present_ptr = self->out_s->p; /* hold 2 byte present pointer, todo */
-  out_uint8s(self->out_s, 2)    /* this can be smaller, */
-                                /* see RDP_ORDER_SMALL and RDP_ORDER_TINY */
+  /* present, set later, 2 bytes */
+  present_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 2);
   if ((order_flags & RDP_ORDER_BOUNDS) &&
       !(order_flags & RDP_ORDER_LASTBOUNDS))
   {
@@ -791,8 +857,8 @@ xrdp_orders_pat_blt(struct xrdp_orders* self, int x, int y,
     g_memcpy(self->orders_state.pat_blt_brush.pattern + 1,
              brush->pattern + 1, 7);
   }
-  present_ptr[0] = present;
-  present_ptr[1] = present >> 8;
+  xrdp_order_pack_small_or_tiny(self, order_flags_ptr, order_flags,
+                                present_ptr, present, 2);
   return 0;
 }
 
@@ -809,6 +875,7 @@ xrdp_orders_dest_blt(struct xrdp_orders* self, int x, int y,
   int vals[8];
   int present;
   char* present_ptr;
+  char* order_flags_ptr;
 
   xrdp_orders_check(self, 21);
   self->order_count++;
@@ -843,14 +910,17 @@ xrdp_orders_dest_blt(struct xrdp_orders* self, int x, int y,
   {
     order_flags |= RDP_ORDER_DELTA;
   }
-  out_uint8(self->out_s, order_flags);
+  /* order_flags, set later, 1 byte */
+  order_flags_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if (order_flags & RDP_ORDER_CHANGE)
   {
-    out_uint8(self->out_s, self->orders_state.last_order)
+    out_uint8(self->out_s, self->orders_state.last_order);
   }
   present = 0;
-  present_ptr = self->out_s->p; /* hold 1 byte present pointer */
-  out_uint8s(self->out_s, 1)
+  /* present, set later, 1 byte */
+  present_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if ((order_flags & RDP_ORDER_BOUNDS) &&
       !(order_flags & RDP_ORDER_LASTBOUNDS))
   {
@@ -914,7 +984,8 @@ xrdp_orders_dest_blt(struct xrdp_orders* self, int x, int y,
     out_uint8(self->out_s, rop);
     self->orders_state.dest_blt_rop = rop;
   }
-  present_ptr[0] = present;
+  xrdp_order_pack_small_or_tiny(self, order_flags_ptr, order_flags,
+                                present_ptr, present, 1);
   return 0;
 }
 
@@ -929,12 +1000,25 @@ xrdp_orders_line(struct xrdp_orders* self, int mix_mode,
                  struct xrdp_pen* pen,
                  struct xrdp_rect* rect)
 {
-  int order_flags;
-  int vals[8];
-  int present;
-  char* present_ptr;
+  int order_flags = 0;
+  int vals[8] = {0, 0, 0, 0, 0, 0, 0, 0};
+  int present = 0;
+  char* present_ptr = (char *)NULL;
+  char* order_flags_ptr = (char *)NULL;
   struct xrdp_pen blank_pen;
 
+  g_memset(&blank_pen,0,sizeof(struct xrdp_pen));
+
+  /* if mix mode or rop are out of range, mstsc build 6000+ will parse the orders
+     wrong */
+  if ((mix_mode < 1) || (mix_mode > 2)) /* TRANSPARENT(1) or OPAQUE(2) */
+  {
+    mix_mode = 1;
+  }
+  if ((rop < 1) || (rop > 0x10))
+  {
+    rop = 0x0d; /* R2_COPYPEN */
+  }
   xrdp_orders_check(self, 32);
   self->order_count++;
   order_flags = RDP_ORDER_STANDARD;
@@ -970,14 +1054,17 @@ xrdp_orders_line(struct xrdp_orders* self, int mix_mode,
   {
     order_flags |= RDP_ORDER_DELTA;
   }
-  out_uint8(self->out_s, order_flags);
+  /* order_flags, set later, 1 byte */
+  order_flags_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if (order_flags & RDP_ORDER_CHANGE)
   {
     out_uint8(self->out_s, self->orders_state.last_order);
   }
   present = 0;
-  present_ptr = self->out_s->p; /* hold 2 byte present pointer */
-  out_uint8s(self->out_s, 2)
+  /* present, set later, 2 bytes */
+  present_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 2);
   if ((order_flags & RDP_ORDER_BOUNDS) &&
       !(order_flags & RDP_ORDER_LASTBOUNDS))
   {
@@ -986,7 +1073,7 @@ xrdp_orders_line(struct xrdp_orders* self, int mix_mode,
   if (mix_mode != self->orders_state.line_mix_mode)
   {
     present |= 0x0001;
-    out_uint16_le(self->out_s, mix_mode)
+    out_uint16_le(self->out_s, mix_mode);
     self->orders_state.line_mix_mode = mix_mode;
   }
   if (startx != self->orders_state.line_startx)
@@ -1044,15 +1131,15 @@ xrdp_orders_line(struct xrdp_orders* self, int mix_mode,
   if (bg_color != self->orders_state.line_bg_color)
   {
     present |= 0x0020;
-    out_uint8(self->out_s, bg_color)
-    out_uint8(self->out_s, bg_color >> 8)
-    out_uint8(self->out_s, bg_color >> 16)
+    out_uint8(self->out_s, bg_color);
+    out_uint8(self->out_s, bg_color >> 8);
+    out_uint8(self->out_s, bg_color >> 16);
     self->orders_state.line_bg_color = bg_color;
   }
   if (rop != self->orders_state.line_rop)
   {
     present |= 0x0040;
-    out_uint8(self->out_s, rop)
+    out_uint8(self->out_s, rop);
     self->orders_state.line_rop = rop;
   }
   if (pen == 0)
@@ -1063,25 +1150,25 @@ xrdp_orders_line(struct xrdp_orders* self, int mix_mode,
   if (pen->style != self->orders_state.line_pen.style)
   {
     present |= 0x0080;
-    out_uint8(self->out_s, pen->style)
+    out_uint8(self->out_s, pen->style);
     self->orders_state.line_pen.style = pen->style;
   }
   if (pen->width != self->orders_state.line_pen.width)
   {
     present |= 0x0100;
-    out_uint8(self->out_s, pen->width)
+    out_uint8(self->out_s, pen->width);
     self->orders_state.line_pen.width = pen->width;
   }
   if (pen->color != self->orders_state.line_pen.color)
   {
     present |= 0x0200;
-    out_uint8(self->out_s, pen->color)
-    out_uint8(self->out_s, pen->color >> 8)
-    out_uint8(self->out_s, pen->color >> 16)
+    out_uint8(self->out_s, pen->color);
+    out_uint8(self->out_s, pen->color >> 8);
+    out_uint8(self->out_s, pen->color >> 16);
     self->orders_state.line_pen.color = pen->color;
   }
-  present_ptr[0] = present;
-  present_ptr[1] = present >> 8;
+  xrdp_order_pack_small_or_tiny(self, order_flags_ptr, order_flags,
+                                present_ptr, present, 2);
   return 0;
 }
 
@@ -1095,10 +1182,11 @@ xrdp_orders_mem_blt(struct xrdp_orders* self, int cache_id,
                     int rop, int srcx, int srcy,
                     int cache_idx, struct xrdp_rect* rect)
 {
-  int order_flags;
-  int vals[12];
-  int present;
-  char* present_ptr;
+  int order_flags = 0;
+  int vals[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+  int present = 0;
+  char* present_ptr = (char *)NULL;
+  char* order_flags_ptr = (char *)NULL;
 
   xrdp_orders_check(self, 30);
   self->order_count++;
@@ -1137,15 +1225,17 @@ xrdp_orders_mem_blt(struct xrdp_orders* self, int cache_id,
   {
     order_flags |= RDP_ORDER_DELTA;
   }
-  out_uint8(self->out_s, order_flags);
+  /* order_flags, set later, 1 byte */
+  order_flags_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if (order_flags & RDP_ORDER_CHANGE)
   {
-    out_uint8(self->out_s, self->orders_state.last_order)
+    out_uint8(self->out_s, self->orders_state.last_order);
   }
   present = 0;
-  present_ptr = self->out_s->p; /* hold 2 byte present pointer, todo */
-  out_uint8s(self->out_s, 2)    /* this can be smaller, */
-                                /* see RDP_ORDER_SMALL and RDP_ORDER_TINY */
+  /* present, set later, 2 bytes */
+  present_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 2);
   if ((order_flags & RDP_ORDER_BOUNDS) &&
       !(order_flags & RDP_ORDER_LASTBOUNDS))
   {
@@ -1250,8 +1340,8 @@ xrdp_orders_mem_blt(struct xrdp_orders* self, int cache_id,
     out_uint16_le(self->out_s, cache_idx);
     self->orders_state.mem_blt_cache_idx = cache_idx;
   }
-  present_ptr[0] = present;
-  present_ptr[1] = present >> 8;
+  xrdp_order_pack_small_or_tiny(self, order_flags_ptr, order_flags,
+                                present_ptr, present, 2);
   return 0;
 }
 
@@ -1268,9 +1358,10 @@ xrdp_orders_text(struct xrdp_orders* self,
                  int x, int y, char* data, int data_len,
                  struct xrdp_rect* rect)
 {
-  int order_flags;
-  int present;
-  char* present_ptr;
+  int order_flags = 0;
+  int present = 0;
+  char* present_ptr = (char *)NULL;
+  char* order_flags_ptr = (char *)NULL;
 
   xrdp_orders_check(self, 100);
   self->order_count++;
@@ -1300,15 +1391,17 @@ xrdp_orders_text(struct xrdp_orders* self,
       }
     }
   }
-  out_uint8(self->out_s, order_flags);
+  /* order_flags, set later, 1 byte */
+  order_flags_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 1);
   if (order_flags & RDP_ORDER_CHANGE)
   {
     out_uint8(self->out_s, self->orders_state.last_order);
   }
   present = 0;
-  present_ptr = self->out_s->p; /* hold 3 byte present pointer, todo */
-  out_uint8s(self->out_s, 3)    /* this can be smaller, */
-                                /* see RDP_ORDER_SMALL and RDP_ORDER_TINY */
+  /* present, set later, 3 bytes */
+  present_ptr = self->out_s->p;
+  out_uint8s(self->out_s, 3);
   if ((order_flags & RDP_ORDER_BOUNDS) &&
       !(order_flags & RDP_ORDER_LASTBOUNDS))
   {
@@ -1336,17 +1429,17 @@ xrdp_orders_text(struct xrdp_orders* self,
   if (fg_color != self->orders_state.text_fg_color)
   {
     present |= 0x000010;
-    out_uint8(self->out_s, fg_color)
-    out_uint8(self->out_s, fg_color >> 8)
-    out_uint8(self->out_s, fg_color >> 16)
+    out_uint8(self->out_s, fg_color);
+    out_uint8(self->out_s, fg_color >> 8);
+    out_uint8(self->out_s, fg_color >> 16);
     self->orders_state.text_fg_color = fg_color;
   }
   if (bg_color != self->orders_state.text_bg_color)
   {
     present |= 0x000020;
-    out_uint8(self->out_s, bg_color)
-    out_uint8(self->out_s, bg_color >> 8)
-    out_uint8(self->out_s, bg_color >> 16)
+    out_uint8(self->out_s, bg_color);
+    out_uint8(self->out_s, bg_color >> 8);
+    out_uint8(self->out_s, bg_color >> 16);
     self->orders_state.text_bg_color = bg_color;
   }
   if (clip_left != self->orders_state.text_clip_left)
@@ -1415,10 +1508,8 @@ xrdp_orders_text(struct xrdp_orders* self,
     out_uint8(self->out_s, data_len);
     out_uint8a(self->out_s, data, data_len);
   }
-  /* send 3 byte present */
-  present_ptr[0] = present;
-  present_ptr[1] = present >> 8;
-  present_ptr[2] = present >> 16;
+  xrdp_order_pack_small_or_tiny(self, order_flags_ptr, order_flags,
+                                present_ptr, present, 3);
   return 0;
 }
 
@@ -1461,14 +1552,14 @@ xrdp_orders_send_raw_bitmap(struct xrdp_orders* self,
                             int width, int height, int bpp, char* data,
                             int cache_id, int cache_idx)
 {
-  int order_flags;
-  int len;
-  int bufsize;
-  int Bpp;
-  int i;
-  int j;
-  int pixel;
-  int e;
+  int order_flags = 0;
+  int len = 0;
+  int bufsize = 0;
+  int Bpp = 0;
+  int i = 0;
+  int j = 0;
+  int pixel = 0;
+  int e = 0;
 
   if (width > 64)
   {
@@ -1541,16 +1632,16 @@ xrdp_orders_send_bitmap(struct xrdp_orders* self,
                         int width, int height, int bpp, char* data,
                         int cache_id, int cache_idx)
 {
-  int order_flags;
-  int len;
-  int bufsize;
-  int Bpp;
-  int i;
-  int lines_sending;
-  int e;
-  struct stream* s;
-  struct stream* temp_s;
-  char* p;
+  int order_flags = 0;
+  int len = 0;
+  int bufsize = 0;
+  int Bpp = 0;
+  int i = 0;
+  int lines_sending = 0;
+  int e = 0;
+  struct stream* s = NULL;
+  struct stream* temp_s = NULL;
+  char* p = NULL;
 
   if (width > 64)
   {
@@ -1583,7 +1674,7 @@ xrdp_orders_send_bitmap(struct xrdp_orders* self,
 height(%d)", lines_sending, height);
     return 1;
   }
-  bufsize = s->p - p;
+  bufsize = (int)(s->p - p);
   Bpp = (bpp + 7) / 8;
   xrdp_orders_check(self, bufsize + 16);
   self->order_count++;
@@ -1632,9 +1723,9 @@ xrdp_orders_send_font(struct xrdp_orders* self,
                       struct xrdp_font_char* font_char,
                       int font_index, int char_index)
 {
-  int order_flags;
-  int datasize;
-  int len;
+  int order_flags = 0;
+  int datasize = 0;
+  int len = 0;
 
   datasize = FONT_DATASIZE(font_char);
   xrdp_orders_check(self, datasize + 18);
@@ -1664,14 +1755,14 @@ xrdp_orders_send_raw_bitmap2(struct xrdp_orders* self,
                              int width, int height, int bpp, char* data,
                              int cache_id, int cache_idx)
 {
-  int order_flags;
-  int len;
-  int bufsize;
-  int Bpp;
-  int i;
-  int j;
-  int pixel;
-  int e;
+  int order_flags = 0;
+  int len = 0;
+  int bufsize = 0;
+  int Bpp = 0;
+  int i = 0;
+  int j = 0;
+  int pixel = 0;
+  int e = 0;
 
   if (width > 64)
   {
@@ -1745,16 +1836,16 @@ xrdp_orders_send_bitmap2(struct xrdp_orders* self,
                          int width, int height, int bpp, char* data,
                          int cache_id, int cache_idx)
 {
-  int order_flags;
-  int len;
-  int bufsize;
-  int Bpp;
-  int i;
-  int lines_sending;
-  int e;
-  struct stream* s;
-  struct stream* temp_s;
-  char* p;
+  int order_flags = 0;
+  int len = 0;
+  int bufsize = 0;
+  int Bpp = 0;
+  int i = 0;
+  int lines_sending = 0;
+  int e = 0;
+  struct stream* s = NULL;
+  struct stream* temp_s = NULL;
+  char* p = NULL;
 
   if (width > 64)
   {
@@ -1787,7 +1878,7 @@ xrdp_orders_send_bitmap2(struct xrdp_orders* self,
 height(%d)", lines_sending, height);
     return 1;
   }
-  bufsize = s->p - p;
+  bufsize = (int)(s->p - p);
   Bpp = (bpp + 7) / 8;
   xrdp_orders_check(self, bufsize + 14);
   self->order_count++;
@@ -1811,3 +1902,31 @@ height(%d)", lines_sending, height);
   free_stream(temp_s);
   return 0;
 }
+
+/*****************************************************************************/
+/* returns error */
+/* send a brush cache entry */
+int APP_CC
+xrdp_orders_send_brush(struct xrdp_orders* self, int width, int height,
+                       int bpp, int type, int size, char* data, int cache_id)
+{
+  int order_flags = 0;
+  int len = 0;
+
+  xrdp_orders_check(self, size + 12);
+  self->order_count++;
+  order_flags = RDP_ORDER_STANDARD | RDP_ORDER_SECONDARY;
+  out_uint8(self->out_s, order_flags);
+  len = (size + 6) - 7; /* length after type minus 7 */
+  out_uint16_le(self->out_s, len);
+  out_uint16_le(self->out_s, 0); /* flags */
+  out_uint8(self->out_s, RDP_ORDER_BRUSHCACHE); /* type */
+  out_uint8(self->out_s, cache_id);
+  out_uint8(self->out_s, bpp);
+  out_uint8(self->out_s, width);
+  out_uint8(self->out_s, height);
+  out_uint8(self->out_s, type);
+  out_uint8(self->out_s, size);
+  out_uint8a(self->out_s, data, size);
+  return 0;
+}
