@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2005-2007
+   Copyright (C) Jay Sorg 2005-2010
 
    librdp rdp layer
 
@@ -22,6 +22,10 @@
 
 #include "rdp.h"
 
+#ifndef NULL
+#define NULL 0
+#endif
+
 /*****************************************************************************/
 struct rdp_rdp* APP_CC
 rdp_rdp_create(struct mod* owner)
@@ -222,7 +226,7 @@ rdp_rdp_out_order_caps(struct rdp_rdp* self, struct stream* s)
 static int APP_CC
 rdp_rdp_out_bmpcache_caps(struct rdp_rdp* self, struct stream* s)
 {
-  int Bpp;
+  int Bpp = 0;
 
   out_uint16_le(s, RDP_CAPSET_BMPCACHE);
   out_uint16_le(s, RDP_CAPLEN_BMPCACHE);
@@ -395,7 +399,7 @@ rdp_rdp_send_confirm_active(struct rdp_rdp* self, struct stream* s)
 
 /******************************************************************************/
 /* Process a color pointer PDU */
-static void APP_CC
+static int APP_CC
 rdp_rdp_process_color_pointer_pdu(struct rdp_rdp* self, struct stream* s)
 {
   int cache_idx;
@@ -404,6 +408,10 @@ rdp_rdp_process_color_pointer_pdu(struct rdp_rdp* self, struct stream* s)
   struct rdp_cursor* cursor;
 
   in_uint16_le(s, cache_idx);
+  if (cache_idx >= sizeof(self->cursors) / sizeof(cursor))
+  {
+    return 1;
+  }
   cursor = self->cursors + cache_idx;
   in_uint16_le(s, cursor->x);
   in_uint16_le(s, cursor->y);
@@ -411,29 +419,39 @@ rdp_rdp_process_color_pointer_pdu(struct rdp_rdp* self, struct stream* s)
   in_uint16_le(s, cursor->height);
   in_uint16_le(s, mlen); /* mask length */
   in_uint16_le(s, dlen); /* data length */
+  if ((mlen > sizeof(cursor->mask)) || (dlen > sizeof(cursor->data)))
+  {
+    return 1;
+  }
   in_uint8a(s, cursor->data, dlen);
   in_uint8a(s, cursor->mask, mlen);
   self->mod->server_set_cursor(self->mod, cursor->x, cursor->y,
                                cursor->data, cursor->mask);
+  return 0;
 }
 
 /******************************************************************************/
 /* Process a cached pointer PDU */
-static void APP_CC
+static int APP_CC
 rdp_rdp_process_cached_pointer_pdu(struct rdp_rdp* self, struct stream* s)
 {
   int cache_idx;
   struct rdp_cursor* cursor;
 
   in_uint16_le(s, cache_idx);
+  if (cache_idx >= sizeof(self->cursors) / sizeof(cursor))
+  {
+    return 1;
+  }
   cursor = self->cursors + cache_idx;
   self->mod->server_set_cursor(self->mod, cursor->x, cursor->y,
                                cursor->data, cursor->mask);
+  return 0;
 }
 
 /******************************************************************************/
 /* Process a system pointer PDU */
-static void APP_CC
+static int APP_CC
 rdp_rdp_process_system_pointer_pdu(struct rdp_rdp* self, struct stream* s)
 {
   int system_pointer_type;
@@ -452,17 +470,20 @@ rdp_rdp_process_system_pointer_pdu(struct rdp_rdp* self, struct stream* s)
     default:
       break;
   }
+  return 0;
 }
 
 /******************************************************************************/
 /* Process a pointer PDU */
-static void APP_CC
+static int APP_CC
 rdp_rdp_process_pointer_pdu(struct rdp_rdp* self, struct stream* s)
 {
   int message_type;
   int x;
   int y;
+  int rv;
 
+  rv = 0;
   in_uint16_le(s, message_type);
   in_uint8s(s, 2); /* pad */
   switch (message_type)
@@ -472,17 +493,18 @@ rdp_rdp_process_pointer_pdu(struct rdp_rdp* self, struct stream* s)
       in_uint16_le(s, y);
       break;
     case RDP_POINTER_COLOR:
-      rdp_rdp_process_color_pointer_pdu(self, s);
+      rv = rdp_rdp_process_color_pointer_pdu(self, s);
       break;
     case RDP_POINTER_CACHED:
-      rdp_rdp_process_cached_pointer_pdu(self, s);
+      rv = rdp_rdp_process_cached_pointer_pdu(self, s);
       break;
     case RDP_POINTER_SYSTEM:
-      rdp_rdp_process_system_pointer_pdu(self, s);
+      rv = rdp_rdp_process_system_pointer_pdu(self, s);
       break;
     default:
       break;
   }
+  return rv;
 }
 
 /******************************************************************************/
@@ -490,26 +512,26 @@ rdp_rdp_process_pointer_pdu(struct rdp_rdp* self, struct stream* s)
 static void APP_CC
 rdp_rdp_process_bitmap_updates(struct rdp_rdp* self, struct stream* s)
 {
-  int num_updates;
-  int left;
-  int top;
-  int right;
-  int bottom;
-  int width;
-  int height;
-  int cx;
-  int cy;
-  int bpp;
-  int Bpp;
-  int compress;
-  int bufsize;
-  int size;
-  int i;
-  int x;
-  int y;
-  char* data;
-  char* bmpdata0;
-  char* bmpdata1;
+  int num_updates = 0;
+  int left = 0;
+  int top = 0;
+  int right = 0;
+  int bottom = 0;
+  int width = 0;
+  int height = 0;
+  int cx = 0;
+  int cy = 0;
+  int bpp = 0;
+  int Bpp = 0;
+  int compress = 0;
+  int bufsize = 0;
+  int size = 0;
+  int i = 0;
+  int x = 0;
+  int y = 0;
+  char* data = NULL;
+  char* bmpdata0 = NULL;
+  char* bmpdata1 = NULL;
 
   in_uint16_le(s, num_updates);
   for (i = 0; i < num_updates; i++)
@@ -563,7 +585,16 @@ rdp_rdp_process_bitmap_updates(struct rdp_rdp* self, struct stream* s)
         {
           for (x = 0; x < width; x++)
           {
-            in_uint16_le(s, ((unsigned short*)data)[x]);
+            in_uint16_le(s, ((tui16*)data)[x]);
+          }
+        }
+        else if (Bpp == 3)
+        {
+          for (x = 0; x < width; x++)
+          {
+            in_uint8(s, data[x * 3 + 0]);
+            in_uint8(s, data[x * 3 + 1]);
+            in_uint8(s, data[x * 3 + 2]);
           }
         }
       }
@@ -606,7 +637,7 @@ rdp_rdp_process_palette(struct rdp_rdp* self, struct stream* s)
 
 /******************************************************************************/
 /* Process an update PDU */
-static void APP_CC
+static int APP_CC
 rdp_rdp_process_update_pdu(struct rdp_rdp* self, struct stream* s)
 {
   int update_type;
@@ -634,6 +665,7 @@ rdp_rdp_process_update_pdu(struct rdp_rdp* self, struct stream* s)
       break;
   }
   self->mod->server_end_update(self->mod);
+  return 0;
 }
 
 
@@ -841,38 +873,38 @@ rdp_rdp_process_data_pdu(struct rdp_rdp* self, struct stream* s)
 {
   int data_pdu_type;
   int ctype;
-  int clen;
   int len;
+  int rv;
 
+  rv = 0;
   in_uint8s(s, 6); /* shareid, pad, streamid */
   in_uint16_le(s, len);
   in_uint8(s, data_pdu_type);
   in_uint8(s, ctype);
-  in_uint16_le(s, clen);
-  clen -= 18;
+  in_uint8s(s, 2); /* clen */
   switch (data_pdu_type)
   {
     case RDP_DATA_PDU_UPDATE:
-      rdp_rdp_process_update_pdu(self, s);
+      rv = rdp_rdp_process_update_pdu(self, s);
       break;
     case RDP_DATA_PDU_CONTROL:
       break;
     case RDP_DATA_PDU_SYNCHRONISE:
       break;
     case RDP_DATA_PDU_POINTER:
-      rdp_rdp_process_pointer_pdu(self, s);
+      rv = rdp_rdp_process_pointer_pdu(self, s);
       break;
-    case RDP_DATA_PDU_BELL:
+    case RDP_DATA_PDU_PLAY_SOUND:
       break;
     case RDP_DATA_PDU_LOGON:
       break;
     case RDP_DATA_PDU_DISCONNECT:
-      rdp_rdp_process_disconnect_pdu(self, s);
+      rv = rdp_rdp_process_disconnect_pdu(self, s);
       break;
     default:
       break;
   }
-  return 0;
+  return rv;
 }
 
 /******************************************************************************/
@@ -887,9 +919,9 @@ rdp_rdp_process_general_caps(struct rdp_rdp* self, struct stream* s)
 static void APP_CC
 rdp_rdp_process_bitmap_caps(struct rdp_rdp* self, struct stream* s)
 {
-  int width;
-  int height;
-  int bpp;
+  int width = 0;
+  int height = 0;
+  int bpp = 0;
 
   in_uint16_le(s, bpp);
   in_uint8s(s, 6);
@@ -905,12 +937,12 @@ rdp_rdp_process_bitmap_caps(struct rdp_rdp* self, struct stream* s)
 static int APP_CC
 rdp_rdp_process_server_caps(struct rdp_rdp* self, struct stream* s, int len)
 {
-  int n;
-  int ncapsets;
-  int capset_type;
-  int capset_length;
-  char* next;
-  char* start;
+  int n = 0;
+  int ncapsets = 0;
+  int capset_type = 0;
+  int capset_length = 0;
+  char* next = NULL;
+  char* start = NULL;
 
   start = s->p;
   in_uint16_le(s, ncapsets);
@@ -1007,9 +1039,9 @@ rdp_rdp_send_fonts(struct rdp_rdp* self, struct stream* s, int seq)
 int APP_CC
 rdp_rdp_process_demand_active(struct rdp_rdp* self, struct stream* s)
 {
-  int type;
-  int len_src_descriptor;
-  int len_combined_caps;
+  int type = 0;
+  int len_src_descriptor = 0;
+  int len_combined_caps = 0;
 
   in_uint32_le(s, self->share_id);
   in_uint16_le(s, len_src_descriptor);
@@ -1036,9 +1068,11 @@ int APP_CC
 rdp_rec_check_file(struct rdp_rdp* self)
 {
   char file_name[256];
-  int index;
-  int len;
-  struct stream* s;
+  int index = 0;
+  int len = 0;
+  struct stream* s = (struct stream *)NULL;
+
+  g_memset(file_name,0,sizeof(char) * 256);
 
   if (self->rec_fd == 0)
   {
@@ -1070,8 +1104,8 @@ rdp_rec_check_file(struct rdp_rdp* self)
 int APP_CC
 rdp_rec_write_item(struct rdp_rdp* self, struct stream* s)
 {
-  int len;
-  int time;
+  int len = 0;
+  int time = 0;
 
   if (self->rec_fd == 0)
   {
